@@ -1,6 +1,7 @@
 import gemmi
 import numpy as np
 import pandas as pd
+from pandas._libs import lib
 from pandas.api.types import is_complex_dtype
 
 from reciprocalspaceship import concat, dtypes
@@ -11,13 +12,15 @@ from reciprocalspaceship.decorators import (
     range_indexed,
     spacegroupify,
 )
-from reciprocalspaceship.dtypes.base import MTZDtype
+from reciprocalspaceship.dtypes.base import MTZDtype, MTZInt32Dtype
 from reciprocalspaceship.utils import (
     apply_to_hkl,
+    assign_with_binedges,
     bin_by_percentile,
     canonicalize_phases,
     compute_structurefactor_multiplicity,
     from_structurefactor,
+    get_reciprocal_grid_size,
     hkl_to_asu,
     hkl_to_observed,
     in_asu,
@@ -141,9 +144,46 @@ class DataSet(pd.DataFrame):
             return self.loc[~self.CENTRIC]
         return self.loc[~self.label_centrics().CENTRIC]
 
+    @property
+    def reindexing_ops(self):
+        """Possible reindexing operations (merohedral twin laws) for DataSet"""
+        return gemmi.find_twin_laws(
+            self.cell, self.spacegroup, max_obliq=1e-6, all_ops=False
+        )
+
     # -------------------------------------------------------------------
     # Methods
 
+    def find_twin_laws(self, max_obliq=1.0, all_ops=False):
+        """
+        Find merohedral and pseudo-merohedral twin laws for cell and spacegroup of
+        DataSet given an obliquity threshold (degrees).
+
+        Notes
+        -----
+        - With `max_obliq=1e-6` and `all_ops=False`, this method returns the same operators
+          as `DataSet.reindexing_ops`
+        - For additional information, see the `GEMMI symmetry page`_.
+
+        .. _GEMMI symmetry page: https://gemmi.readthedocs.io/en/latest/symmetry.html#twinning
+
+        Parameters
+        ----------
+        max_obliq : float
+            Obliquity threshold (in degrees) as defined in Le Page, J Appl Cryst (1982).
+            (default: 1.0)
+        all_ops : bool
+            Whether to return all twin operators. If False, only non-redundant operators
+            are returned (coset representative).
+
+        Returns
+        -------
+        List of gemmi.Op
+        """
+        return gemmi.find_twin_laws(
+            self.cell, self.spacegroup, max_obliq=max_obliq, all_ops=all_ops
+        )
+
     @inplace
     def _index_from_names(self, names, inplace=False):
         """Helper method for decorators"""
@@ -291,6 +331,55 @@ class DataSet(pd.DataFrame):
             dataset = _handle_cached_dtypes(dataset, columns, drop)
             return dataset
 
+    def to_numpy(self, dtype=None, copy=False, na_value=lib.no_default):
+        """
+        Convert the DataSet to a NumPy array.
+
+        This method will attempt to infer a consensus numpy dtype from the dtypes
+        of the DataSet columns. If the DataSet is composed of all `int32`-backed
+        MTZ dtypes and does contain NaN values, the returned `dtype` will be `int32`.
+        For all other combinations of `MTZDtype`, the returned dtype will be `float32`.
+        If the DataSet contains dtypes other than `MTZDtype`, the default Pandas
+        behavior is used (see `Pandas documentation`_).
+
+        .. _Pandas documentation: https://pandas.pydata.org/docs/reference/api/pandas.DataFrame.to_numpy.html
+
+        Parameters
+        ----------
+        dtype : str or np.dtype
+            The dtype to pass to `np.asarray()`
+        copy : bool
+            Whether to ensure that the returned value is not a view on another array.
+            Note that `copy=False` does not ensure that `to_numpy()` is no-copy. Rather,
+            `copy=True` ensure that a copy is made, even if not strictly necessary.
+            (default: False)
+        na_value : Any
+            The value to use for missing values. The default value depends on `dtype` and
+            the dtypes of the DataSet columns.
+
+        Returns
+        -------
+        np.ndarray
+        """
+        if dtype is None:
+            dtype_list = [self.dtypes[k] for k in self]
+
+            # If all dtypes are MTZInt32Dtype, we can coerce to either int32 or float32
+            if all([isinstance(d, MTZInt32Dtype) for d in dtype_list]):
+                if not any([self[k].hasnans for k in self]):
+                    return super().to_numpy(dtype="int32", copy=copy, na_value=na_value)
+                else:
+                    return super().to_numpy(
+                        dtype="float32", copy=copy, na_value=na_value
+                    )
+
+            # All MTZDtypes can be represented as float32
+            if all([isinstance(d, MTZDtype) for d in dtype_list]):
+                return super().to_numpy(dtype="float32", copy=copy, na_value=na_value)
+
+        # Use Pandas default behavior
+        return super().to_numpy(dtype=dtype, copy=copy, na_value=na_value)
+
     @classmethod
     def from_gemmi(cls, gemmiMtz):
         """
@@ -790,33 +879,70 @@ class DataSet(pd.DataFrame):
         return self
 
     @inplace
-    def assign_resolution_bins(self, bins=20, inplace=False, return_labels=True):
+    def assign_resolution_bins(
+        self,
+        bins=20,
+        inplace=False,
+        return_labels=True,
+        format_str=".2f",
+        return_edges=False,
+    ):
         """
         Assign reflections in DataSet to resolution bins.
 
+        Notes
+        -----
+        - If bin edges are provided, any reflections outside of the specified range
+          are dropped.
+
         Parameters
         ----------
-        bins : int
-            Number of bins
+        bins : int, list, or np.ndarray
+            Number of bins or bin edges to use when assigning resolution bins. If
+            bin edges are provided, they must be monotonic (default: 20)
         inplace : bool
-            Whether to add the column in place or return a copy
+            Whether to add the column in place or return a copy (default: False)
         return_labels : bool
             Whether to return a list of labels corresponding to the edges
-            of each resolution bin
+            of each resolution bin (default: True)
+        format_str : str
+            Format string for constructing bin labels
+        return_edges : bool
+            Whether to return bin edges that define the resolution bin boundaries.
+            The bin edges are returned as a 1-dimensional array with `bins + 1` entries
+            (default: False)
 
         Returns
         -------
-        (DataSet, list) or DataSet
+        (DataSet, list), (DataSet, ndarray), (DataSet, list, ndarray) or DataSet
         """
         dHKL = self.compute_dHKL()["dHKL"]
 
-        assignments, labels = bin_by_percentile(dHKL, bins=bins, ascending=False)
-        self["bin"] = DataSeries(assignments, dtype="I", index=self.index)
-
-        if return_labels:
-            return self, labels
+        if isinstance(bins, int):
+            assignments, edges = bin_by_percentile(dHKL, bins=bins, ascending=False)
         else:
+            # Use bin edges for assignments and drop reflections outside of range
+            mask = (dHKL >= min(bins)) & (dHKL <= max(bins))
+            assignments = assign_with_binedges(dHKL[mask], bin_edges=bins)
+            edges = np.array(bins)
+            self._update_inplace(self.loc[mask])
+
+        self.loc[:, "bin"] = DataSeries(assignments, dtype="I", index=self.index)
+
+        # Package return values
+        result = [self]
+        if return_labels:
+            labels = [
+                f"{e1:{format_str}} - {e2:{format_str}}"
+                for e1, e2 in zip(edges[:-1], edges[1:])
+            ]
+            result.append(labels)
+        if return_edges:
+            result.append(edges)
+
+        if len(result) == 1:
             return self
+        return tuple(result)
 
     def stack_anomalous(
         self, plus_labels=None, minus_labels=None, suffixes=("(+)", "(-)")
@@ -1287,31 +1413,162 @@ class DataSet(pd.DataFrame):
 
         return self
 
-    def to_reciprocalgrid(self, key, gridsize):
+    def get_reciprocal_grid_size(self, dmin=None, sample_rate=3.0):
+        """
+        Determine an appropriate 3D grid size for reflection data.
+
+        Returns the smallest grid size that yields a real-space grid spacing
+        of at most `dmin`/`sample_rate` (in Å). The returned grid size will
+        be 'FFT-friendly' (2, 3, or 5 are the largest prime factors), and will
+        obey any symmetry constraints of the spacegroup.
+
+        Parameters
+        ----------
+        dmin : float
+            Highest-resolution reflection to consider for grid size
+        sample_rate : float
+            Sets the minimal grid spacing relative to dmin. For example,
+            `sample_rate=3` corresponds to a real-space sampling of dmin/3.
+            Value must be >= 1.0 (default: 3.0)
+
+        Returns
+        -------
+        list(int, int, int)
+            Grid size with desired spacing (list of 3 integers)
+        """
+        if dmin is None:
+            dmin = self.compute_dHKL().dHKL.min()
+
+        return get_reciprocal_grid_size(
+            self.cell, dmin=dmin, sample_rate=sample_rate, spacegroup=self.spacegroup
+        )
+
+    def to_reciprocal_grid(self, key, sample_rate=3.0, dmin=None, grid_size=None):
         """
         Set up reciprocal grid with values from column, ``key``, indexed by
-        Miller indices. A 3D numpy array will be initialized with the
-        specified gridsize. Any missing Miller indices are initialized to
-        zero.
+        Miller indices.
 
         Notes
         -----
         - The data being arranged on a reciprocal grid must be compatible
           with a numpy datatype.
+        - Any missing Miller indices are initialized to zero.
+        - If explicitly provided, `grid_size` supersedes `dmin` and `sample_rate`
+          for grid size determination.
+        - The grid size determined using `sample_rate` and `dmin` will depend
+          on the cell parameters of the dataset. If the grid size must be
+          consistent across different isomorphous cell parameters, `grid_size`
+          can be explicitly provided.
 
         Parameters
         ----------
         key : str
             Column label for value to arrange on reciprocal grid
-        gridsize : array-like (len==3)
-            Dimensions for 3D reciprocal grid.
+        sample_rate : float
+            Sets the minimal grid spacing relative to dmin. For example,
+            `sample_rate=3` corresponds to a real-space sampling of dmin/3.
+            (default: 3.0)
+        dmin : float
+            Highest-resolution reflection to consider for grid size. If None,
+            `dmin` will be set to the highest resolution reflection in the
+            dataset. The reflections used to populate the grid will also
+            be truncated to `dHKL >= dmin` (default: None)
+        grid_size : array-like (len==3)
+            If given, provides the explicit dimensions for 3D reciprocal
+            grid. If None, grid size will be set based on `sample_rate` and
+            `dmin`. If provided, this grid size will be used regardless of
+            the values provided as `sample_rate` and `dmin`
 
         Returns
         -------
         numpy.ndarray
         """
+        if dmin is not None:
+            ds = self.loc[self.compute_dHKL().dHKL >= dmin, [key]]
+        else:
+            ds = self.loc[:, [key]]
+
+        if grid_size is None:
+            grid_size = self.get_reciprocal_grid_size(
+                dmin=dmin, sample_rate=sample_rate
+            )
+
         # Set up P1 unit cell
-        p1 = self.expand_to_p1()
+        p1 = ds.expand_to_p1()
+        p1 = p1.expand_anomalous()
+
+        # Get data and indices
+        data = p1[key].to_numpy()
+        H = p1.get_hkls()
+
+        # Populate grid
+        grid = np.zeros(grid_size, dtype=data.dtype)
+        grid[H[:, 0], H[:, 1], H[:, 2]] = data
+
+        return grid
+
+    def to_reciprocalgrid(self, key, sample_rate=3.0, dmin=None, gridsize=None):
+        """
+        Deprecated: Set up reciprocal grid with values from column, ``key``, indexed by
+        Miller indices.
+
+        .. warning:: This function is deprecated. Use ``to_reciprocal_grid()`` instead.
+
+        Notes
+        -----
+        - The data being arranged on a reciprocal grid must be compatible
+          with a numpy datatype.
+        - Any missing Miller indices are initialized to zero.
+        - If explicitly provided, `gridsize` supersedes `dmin` and `sample_rate`
+          for grid size determination.
+        - The grid size determined using `sample_rate` and `dmin` will depend
+          on the cell parameters of the dataset. If the grid size must be
+          consistent across different isomorphous cell parameters, `gridsize`
+          can be explicitly provided.
+
+        Parameters
+        ----------
+        key : str
+            Column label for value to arrange on reciprocal grid
+        sample_rate : float
+            Sets the minimal grid spacing relative to dmin. For example,
+            `sample_rate=3` corresponds to a real-space sampling of dmin/3.
+            (default: 3.0)
+        dmin : float
+            Highest-resolution reflection to consider for grid size. If None,
+            `dmin` will be set to the highest resolution reflection in the
+            dataset. The reflections used to populate the grid will also
+            be truncated to `dHKL >= dmin` (default: None)
+        gridsize : array-like (len==3)
+            If given, provides the explicit dimensions for 3D reciprocal
+            grid. If None, grid size will be set based on `sample_rate` and
+            `dmin`. If provided, this grid size will be used regardless of
+            the values provided as `sample_rate` and `dmin`
+
+        Returns
+        -------
+        numpy.ndarray
+        """
+        import warnings
+
+        message = (
+            "DataSet.to_reciprocalgrid(key, gridsize=...) has been renamed to "
+            "DataSet.to_reciprocal_grid(key... grid_size=None).\n"
+            "DataSet.to_reciprocalgrid() will be removed in a future release."
+        )
+        warnings.simplefilter("always")
+        warnings.warn(message, DeprecationWarning)
+
+        if dmin is not None:
+            ds = self.loc[self.compute_dHKL().dHKL >= dmin, [key]]
+        else:
+            ds = self.loc[:, [key]]
+
+        if gridsize is None:
+            gridsize = self.get_reciprocal_grid_size(dmin=dmin, sample_rate=sample_rate)
+
+        # Set up P1 unit cell
+        p1 = ds.expand_to_p1()
         p1 = p1.expand_anomalous()
 
         # Get data and indices
