@@ -8,10 +8,13 @@ __all__ = [
     "is_centric",
     "is_absent",
     "apply_to_hkl",
+    "is_polar",
+    "polar_axes",
     "phase_shift",
     "add_rfree",
     "copy_rfree",
     "compute_dHKL",
+    "get_reciprocal_grid_size",
     "generate_reciprocal_cell",
     "hkl_to_asu",
     "hkl_to_observed",
@@ -31,10 +34,11 @@ from reciprocalspaceship.utils.asu import (
 )
 from reciprocalspaceship.utils.binning import assign_with_binedges, bin_by_percentile
 from reciprocalspaceship.utils.cell import compute_dHKL, generate_reciprocal_cell
+from reciprocalspaceship.utils.grid import get_reciprocal_grid_size
 from reciprocalspaceship.utils.math import angle_between
 from reciprocalspaceship.utils.phases import canonicalize_phases, get_phase_restrictions
 from reciprocalspaceship.utils.rfree import add_rfree, copy_rfree
-from reciprocalspaceship.utils.stats import compute_redundancy
+from reciprocalspaceship.utils.stats import compute_redundancy, weighted_pearsonr
 from reciprocalspaceship.utils.structurefactors import (
     compute_structurefactor_multiplicity,
     from_structurefactor,
@@ -42,5 +46,10 @@ from reciprocalspaceship.utils.structurefactors import (
     is_centric,
     to_structurefactor,
 )
-from reciprocalspaceship.utils.symop import apply_to_hkl, phase_shift
+from reciprocalspaceship.utils.symmetry import (
+    apply_to_hkl,
+    is_polar,
+    phase_shift,
+    polar_axes,
+)
 from reciprocalspaceship.utils.units import angstroms2ev, ev2angstroms
