@@ -417,35 +417,6 @@ def test_compute_multiplicity(dataset_hkl, inplace, include_centering, spacegrou
     assert isinstance(result["EPSILON"].dtype, rs.MTZIntDtype)
 
 
-@pytest.mark.parametrize("bins", [5, 10, 20, 50])
-@pytest.mark.parametrize("inplace", [True, False])
-@pytest.mark.parametrize("return_labels", [True, False])
-def test_assign_resolution_bins(data_fmodel, bins, inplace, return_labels):
-    """Test DataSet.assign_resolution_bins"""
-
-    result = data_fmodel.assign_resolution_bins(
-        bins=bins, inplace=inplace, return_labels=return_labels
-    )
-
-    if return_labels:
-        result, labels = result
-
-    # Test bins
-    assert "bin" in result.columns
-    assert len(result["bin"].unique()) == bins
-    assert result.bin.max() == bins - 1
-
-    # Test inplace
-    if inplace:
-        assert id(result) == id(data_fmodel)
-    else:
-        assert id(result) != id(data_fmodel)
-
-    # Test labels
-    if return_labels:
-        assert len(labels) == bins
-
-
 @pytest.mark.parametrize("inplace", [True, False])
 @pytest.mark.parametrize(
     "op",
